import { Component } from '@angular/core';
import { FormControl } from '@angular/forms';
import { AppService, WindowSize } from './modules/shared/services/app/app.service';
import { ThemeService } from './modules/shared/services/theme/theme.service';

enum SidebarOpenEnum {
  Default,
  Open,
  Closed,
}
@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent {
  isMobile = true;
  title = 'CureNet';
  sidebarOpen = SidebarOpenEnum.Default;
  sidebarMode = 'over';
  defaultSidebarOpen = false;
  lang: string;
  theme: string;
  langs: string[] = [];
  themes: string[] = [];
  themeControl = new FormControl();
  langControl = new FormControl();
  isDark = true;

  get isSidebarOpen() {
    switch (this.sidebarOpen) {
      case SidebarOpenEnum.Open:
        return true;
        break;
      case SidebarOpenEnum.Closed:
        return false;
        break;
      default:
        return this.defaultSidebarOpen;
        break;
    }
  }
  constructor(
    appService: AppService,
    themeService: ThemeService,
  ) {
    this.themes = themeService.getThemes();
    this.langs = appService.getLangs();
    this.lang = appService.getLang();

    this.themeControl.valueChanges.subscribe(theme => {
      themeService.setTheme(theme);
    });
    this.langControl.valueChanges.subscribe(lang => {
      appService.changeLang(lang);
    });
    this.onThemeChanged(themeService.getTheme());
    themeService.themeChange.subscribe(theme => {
      this.onThemeChanged(theme);
    });
    this.onResize(appService.size);
    appService.sizeChange.subscribe(size => {
      this.onResize(size);
    });
    this.onLangChanged(appService.getLang());
    appService.langChange.subscribe(lang => {
      this.onLangChanged(lang);
    });
  }

  onThemeChanged(theme: string) {
    if (theme === undefined) {
      return;
    }
    this.theme = theme;
    this.themeControl.setValue(theme);
    this.isDark = theme.indexOf('dark') !== -1;
  }

  onResize(size: WindowSize) {
    if (size === undefined) {
      return;
    }
    this.isMobile = size.isMobile;
    this.defaultSidebarOpen = !this.isMobile;
    this.sidebarMode = this.isMobile ? 'over' : 'side';
  }

  onLangChanged(lang: string) {
    if (lang === undefined) {
      return;
    }
    this.lang = lang;
    this.langControl.setValue(lang);
  }

  toggleSidebar() {
    this.sidebarOpen = this.isSidebarOpen ? SidebarOpenEnum.Closed : SidebarOpenEnum.Open;
  }

  onSidebarOpenedChange(opened: boolean) {
    this.sidebarOpen = opened ? SidebarOpenEnum.Open : SidebarOpenEnum.Closed;
  }
}